@@ -1,10 +1,11 @@
 # 2014. 12. 9 by Hans Roh hansroh@gmail.com
 
-__version__ = "0.30.0.1"
+__version__ = "0.33.3.4"
 
 version_info = tuple (map (lambda x: not x.isdigit () and x or int (x),  __version__.split (".")))
 NAME = "Skitai/%s.%s" % version_info [:2]
 
+import aquests # should be first for psycopg2 compat
 from rs4 import deco, importer
 from rs4.psutil import service
 from rs4.attrdict import AttrDict
@@ -18,24 +19,36 @@ from aquests.dbapi import (
 import warnings
 from aquests.protocols.smtp import composer
 import tempfile
-from . import lifetime
+from rs4 import argopt
+from .backbone import lifetime
 from . import mounted
 from .corequest import corequest
 from functools import wraps
 import copy
 import rs4
+from rs4.termcolor import tc
 
-rs4.add_options (
-    "-d", # daemonize
-    "---profile", "---gc", "---memtrack", # dev options
-    "--production", "--smtpda", "--port="
-)
+argopt.add_option ('-d', desc = "start as daemon, equivalant with using `start` command") # lower version compatible
+argopt.add_option (None, '---profile', desc = "log for performance profiling")
+argopt.add_option (None, '---gc', desc = "enable manual GC")
+argopt.add_option (None, '---memtrack', desc = "show memory status")
+
+argopt.add_option (None, '--production', desc = "run as production mode")
+argopt.add_option (None, '--smtpda', desc = "run SMTPDA if not started")
+argopt.add_option (None, '--port=PORT_NUMBER', desc = "http/https port number")
+argopt.add_option (None, '--quic=UDP_PORT_NUMBER', desc = "http3/quic port number")
 
 if "--production" in sys.argv:
     os.environ ["SKITAI_ENV"] = "PRODUCTION"
 
+SMTP_STARTED = False
 if "--smtpda" in sys.argv:
-    os.system ("{} -m skitai.bin.skitai smtpda -d".format (sys.executable))
+    os.system ("{} -m skitai.scripts.skitai smtpda -d".format (sys.executable))
+    SMTP_STARTED = True
+
+def set_smtp (server, user = None, password = None, ssl = False, start_service = True):
+    composer.set_default_smtp (server, user, password, ssl)
+    start_service and not SMTP_STARTED and os.system ("{} -m skitai.scripts.skitai smtpda -d".format (sys.executable))
 
 def test_client (*args, **kargs):
     from .testutil.launcher import Launcher
@@ -69,7 +82,7 @@ WEBSOCKET_GROUPCHAT = 5
 
 WS_CHANNEL = WS_SIMPLE = 1
 WS_GROUPCHAT = 5
-# WS_THREADSAFE = 6 => 134
+WS_THREADSAFE_DEPRECATED = 7
 
 # optional executing ways
 WS_THREAD = 0
@@ -165,9 +178,7 @@ def detect_atila ():
 
 def websocket (varname = 60, timeout = 60, onopen = None, onclose = None):
     global was
-    if not isinstance (varname, int):
-        warnings.warn ("varname argument has been deprecated, use websokect (timeout)", DeprecationWarning)
-    else:
+    if isinstance (varname, int):
         assert not onopen and not onclose, 'skitai.WS_SESSION cannot have onopen or onclose handler'
         timeout, varname = varname, None
 
@@ -212,18 +223,28 @@ def manual_gc (interval = 60.0):
     lifetime.manual_gc (interval)
 
 def set_worker_critical_point (cpu_percent = 90.0, continuous = 3, interval = 20):
-    from .http_server import http_server
-    from .https_server import https_server
+    from .backbone.http_server import http_server
+    from .backbone.https_server import https_server
 
     http_server.critical_point_cpu_overload = https_server.critical_point_cpu_overload = cpu_percent
     http_server.critical_point_continuous = https_server.critical_point_continuous = continuous
     http_server.maintern_interval = https_server.maintern_interval = interval
 
+
 class Preference (AttrDict):
-    def __init__ (self):
-        super (Preference, self).__init__ (self)
+    def __init__ (self, path = None):
+        super ().__init__ ()
+        self.__path = path
         self.__dict__ ["mountables"] = []
 
+    def __enter__ (self):
+        if self.__path:
+            sys.path.insert (0, joinpath (self.__path))
+        return self
+
+    def __exit__ (self, *args):
+        pass
+
     def mount (self, *args, **kargs):
         self.__dict__ ["mountables"].append ((args, kargs))
 
@@ -231,9 +252,9 @@ class Preference (AttrDict):
         return copy.deepcopy (self)
 
 
-def preference (preset = False):
-    from .wsgi_apps import Config
-    d = Preference ()
+def preference (preset = False, path = None):
+    from .wastuff.wsgi_apps import Config
+    d = Preference (path)
     d.config = Config (preset)
     return d
 pref = preference
@@ -359,14 +380,24 @@ def maybe_django (wsgi_path, appname):
         sys.path.insert (0, root)
         return root
 
-def mount (point, target, appname = "app", pref = pref (True), host = "default", path = None):
+def mount (point, target, __some = pref (True), *args, **kargs):
+    if isinstance (__some, Preference):
+        if 'pref' not in kargs:
+            kargs ['pref'] = __some
+        return _mount (point, target, 'app', *args, **kargs)
+    return _mount (point, target, __some, *args, **kargs)
+
+def _mount (point, target, appname = "app", pref = pref (True), host = "default", path = None):
     global dconf
 
     if isinstance (appname, Preference):
         pref, appname = appname, "app"
 
     def init_app (modpath, pref):
-        modinit = os.path.join (os.path.dirname (modpath), "__init__.py")
+        srvice_root = os.path.dirname (modpath)
+        # IMP: MUST pathing because reloading module
+        sys.path.append (srvice_root)
+        modinit = os.path.join (srvice_root, "__init__.py")
         if os.path.isfile (modinit):
             mod = importer.from_file ("temp", modinit)
             hasattr (mod, "bootstrap") and mod.bootstrap (pref)
@@ -404,16 +435,28 @@ def mount (point, target, appname = "app", pref = pref (True), host = "default",
     if os.path.isdir (target) or not appname:
         dconf ['mount'][host].append ((point, target, None))
     else:
+        target_ = target
+        if not target_.endswith ('.py'):
+            target_ += '.py'
+        if 'PYTEST_CURRENT_TEST' not in os.environ:
+            assert os.path.exists (target_),  'app not found: {}'.format (target_)
+            with open (target_) as f:
+                if f.read ().find ("atila") != -1:
+                    try:
+                        import atila # automatic patch skitai was
+                    except ImportError:
+                        pass
         init_app (target, pref)
         dconf ['mount'][host].append ((point,  (target, appname), pref))
 
 mount_django = mount
 
-def enable_forward (forward_to = 443, port = 80, ip = ""):
+def enable_forward (port = 80, forward_port = 443, forward_domain = None, ip = ""):
     global dconf
     dconf ['fws_address'] = ip
-    dconf ['fws_port'] = forward_to
-    dconf ['fws_to'] = port
+    dconf ['fws_port'] = port
+    dconf ['fws_to'] = forward_port
+    dconf ['fws_domain'] = forward_domain
 
 def enable_gateway (enable_auth = False, secure_key = None, realm = "Skitai API Gateway"):
     global dconf
@@ -495,7 +538,7 @@ def enable_blacklist (path):
     global dconf
     dconf ["blacklist_dir"] = path
 
-def enable_ssl (certfile, keyfile, passphrase):
+def enable_ssl (certfile, keyfile = None, passphrase = None):
     global dconf
     dconf ["certfile"] = certfile
     dconf ["keyfile"] = keyfile
@@ -510,35 +553,41 @@ def get_logpath (name):
     return os.name == "posix" and '/var/log/skitai/%s' % name or os.path.join (tempfile.gettempdir(), name)
 
 options = None
+def add_option (sopt, lopt = None, desc = None):
+    global options
+    argopt.add_option (sopt, lopt, desc)
+    options = argopt.options ()
+
 def add_options (*lnames):
     global options
-
+    # deprecated, use add_option for detail description
     for lname in lnames:
         assert lname and lname [0] == "-", "Aurgument should start with '-' or '--'"
         assert lname != "-d" and lname != "-d=", "Aurgument -d is in ussed"
         if lname.startswith ("--"):
-            rs4.add_option (lname [2:])
+            argopt.add_option (None, lname [2:])
         else:
-            rs4.add_option (None, lname [1:])
-    options = rs4.options ()
+            argopt.add_option (lname [1:])
+    options = argopt.options ()
 
 def getopt (sopt = "", lopt = []):
     global options
 
+    # argopt.getopt style
     if "d" in sopt:
         raise SystemError ("-d is used by skitai, please change")
     for each in lopt:
-        rs4.add_option (each, None)
+        argopt.add_option (None, each)
 
     grps = sopt.split (":")
     for idx, grp in enumerate (grps):
         for idx2, each in enumerate (grp):
             if idx2 == len (grp) - 1 and len (grps) > idx + 1:
-                rs4.add_option (None, each + ":")
+                argopt.add_option (each + ":")
             else:
-                rs4.add_option (None, each)
+                argopt.add_option (each)
 
-    options = rs4.options ()
+    options = argopt.options ()
     opts_ = []
     for k, v in options.items ():
         if k == "-d":
@@ -553,7 +602,25 @@ def getopt (sopt = "", lopt = []):
             continue
         aopt_.append (arg)
     return opts_, aopt_
-argopt = getopt
+
+def get_command ():
+    global options
+    options = argopt.options ()
+    if '--help' in options:
+        print ("{}: {} [OPTION]... [COMMAND]...".format (tc.white ("Usage"), sys.argv [0]))
+        print ("COMMAND can be one of [status|start|stop|restart]")
+        argopt.usage ()
+        sys.exit ()
+
+    cmd = None
+    if "-d" in options:
+        cmd = "start"
+    else:
+        for cmd_ in ("start", "stop", "status", "restart"):
+            if cmd_ in options.argv:
+                cmd = cmd_
+                break
+    return cmd
 
 def getsysopt (name, default = None):
     try:
@@ -564,17 +631,6 @@ def getsysopt (name, default = None):
 def hassysopt (name):
     return "---{}".format (name) in sys.argv
 
-def get_command ():
-    opts = rs4.options ()
-    cmd = None
-    if "d" in opts:
-        cmd = "start"
-    else:
-        for cmd_ in ("start", "stop", "status", "restart"):
-            if cmd_ in opts.argv:
-                cmd = cmd_
-                break
-    return cmd
 
 def sched (interval, func):
     lifetime.maintern.sched (interval, func)
@@ -584,7 +640,6 @@ def run (**conf):
     from . import Skitai
     from rs4.psutil import flock
     from rs4 import pathtool
-    import getopt
 
     class SkitaiServer (Skitai.Loader):
         NAME = 'instance'
@@ -613,9 +668,9 @@ def run (**conf):
 
         def master_jobs (self):
             if os.environ.get ("SKITAI_ENV") == "PRODUCTION":
-                self.wasc.logger ("server", "[info] running for production mode")
+                self.wasc.logger ("server", "[info] running in production mode")
             else:
-                self.wasc.logger ("server", "[info] running for development mode")
+                self.wasc.logger ("server", "[info] running in development mode")
             self.wasc.logger ("server", "[info] engine tmp path: %s" % self.varpath)
             if self.logpath:
                 self.wasc.logger ("server", "[info] engine log path: %s" % self.logpath)
@@ -638,10 +693,7 @@ def run (**conf):
             self.flock.unlock ("signal")
 
         def configure (self):
-            global options
-
-            if options is None:
-                options = rs4.options ()
+            options = argopt.options ()
 
             conf = self.conf
             self.set_num_worker (conf.get ('workers', 1))
@@ -662,15 +714,19 @@ def run (**conf):
             self.config_rcache (conf.get ("rcache_objmax", 100))
             if conf.get ('fws_to'):
                 self.config_forward_server (
-                    conf.get ('fws_address', '0.0.0.0'), conf.get ('fws_port', 80), conf.get ('fws_to', 443)
+                    conf.get ('fws_address', '0.0.0.0'),
+                    conf.get ('fws_port', 80), conf.get ('fws_to', 443)
                 )
 
             port = int (options.get ('--port') or conf.get ('port', 5000))
+            quic = int (options.get ('--quic') or conf.get ('quic', 0))
             self.config_webserver (
                 port, conf.get ('address', '0.0.0.0'),
                 NAME, conf.get ("certfile") is not None,
                 conf.get ('keep_alive', DEFAULT_KEEP_ALIVE),
                 conf.get ('network_timeout', DEFAULT_NETWORK_TIMEOUT),
+                conf.get ('fws_domain'),
+                quic = quic,
                 thunks = [self.master_jobs]
             )
 
