@@ -4,7 +4,7 @@ import os
 
 class SelectiveLogger:
 	MAX_CACHE = 1000
-	
+
 	def __init__ (self, log_off = []):
 		self.log_off = log_off
 		self.endswiths = []
@@ -16,42 +16,46 @@ class SelectiveLogger:
 				self.endswiths.append (each [1:])
 			elif each.endswith ("*"):
 				self.startswiths.append (each[:-1])
-			else:				
-				self.startswiths.append (each)		
-	
-	def add_cache (self, key, val):	
+			else:
+				self.startswiths.append (each)
+
+	def add_cache (self, key, val):
 		with self.lock:
 			if len (self.cache) > self.MAX_CACHE:
 				return
 			self.cache [key] = val
-		
-	def __call__ (self, uri, producer, logfunc):
+
+	def loggable (self, uri):
 		if not self.log_off or uri == "/":
-			return producers.hooked_producer (producer, logfunc)		
-		
+			return True
+
 		with self.lock:
 			cached = self.cache.get (uri)
 		if cached == -1:
-			return producer
-		
+			return False
+
 		for each in self.endswiths:
 			if uri.endswith (each):
 				self.add_cache (uri, -1)
-				return producer
-				
+				return False
+
 		d = os.path.dirname (uri)
 		with self.lock:
 			cached = self.cache.get (d)
 		if cached == -1:
-			return producer
+			return False
 		elif cached == 1:
-			return producers.hooked_producer (producer, logfunc)
-		
+			return True
+
 		for each in self.startswiths:
 			if uri.startswith (each):
 				self.add_cache (d, -1)
-				return producer
+				return False
 
 		self.add_cache (d, 1)
-		return producers.hooked_producer (producer, logfunc)		
-		
\ No newline at end of file
+		return True
+
+	def __call__ (self, uri, producer, logfunc):
+		if self.loggable (uri):
+			return producers.hooked_producer (producer, logfunc)
+		return producer
